@@ -208,6 +208,11 @@ class GajimApplication(Gtk.Application, CoreApplication):
         return [option]
 
     def _startup(self, _application: GajimApplication) -> None:
+        if sys.platform == 'win32':
+            # Changing the PANGOCAIRO_BACKEND is necessary on Windows
+            # to render colored emoji glyphs
+            os.environ['PANGOCAIRO_BACKEND'] = 'fontconfig'
+
         self._init_core()
 
         icon_theme = Gtk.IconTheme.get_default()
@@ -227,7 +232,9 @@ class GajimApplication(Gtk.Application, CoreApplication):
         idle.Monitor.set_interval(app.settings.get('autoawaytime') * 60,
                                   app.settings.get('autoxatime') * 60)
 
-        if sys.platform in ('win32', 'darwin'):
+        if sys.platform == 'darwin':
+            # TODO: Remove if colored emoji rendering works well on
+            # Windows and MacOS
             from gajim.gui.emoji_chooser import emoji_chooser
             emoji_chooser.load()
 
@@ -806,7 +813,6 @@ class GajimApplication(Gtk.Application, CoreApplication):
         client.get_module('Bookmarks').remove(params.jid)
 
         app.storage.archive.remove_history(params.account, params.jid)
-        app.storage.archive.forget_jid_data(params.account, params.jid)
 
     @staticmethod
     def _on_groupchat_join_action(_action: Gio.SimpleAction,
